#--
# gott.py -- use Gott's ideas to estimate traversal times
# J. A. Templon, NIKHEF, PDP Group
# $Id: gott.py,v 1.2 2005/07/27 08:24:18 templon Exp $
#--

# this version still hard codes an awful lot.

# there are several algorithms for what to do with queued jobs.
# define them here and provide means to select between them.

def ett_longest_queue_time(server,list) :
    est = 999
    qtlist = [ ]
    
    for j in list:
        if j.get('state') == 'queued' :
            qtlist.append(server.now() - j.get('qtime'))
    qtlist.sort()
    est = qtlist[-1]   # sorted list so this is the longest waiting.

    return est

def ett_avg_queue_time(server,list) :
    est = 999
    # find average time in queue of all queued jobs
    count = 0
    sum = 0.0
    for j in list:
        if j.get('state') == 'queued' :
            count = count + 1
            sum = sum + (server.now() - j.get('qtime'))
    avgtime = sum / count
    # avg time in queue likely to be half of expected time in queue
    # => return twice avg as estimate.
    est = int(2*avgtime)

    return est

# make dict of functions to allow run-time selection
_ALGS = {
    'average' : ett_avg_queue_time,
    'longest' : ett_longest_queue_time }

# adjusted_ett provides a central place to make any changes to
# return values; right now it just puts a lower limit of 5 on
# the lower limit.  Could be used e.g. to change units.
def adjusted_ett(rawval):
    return max(rawval,5)

def ett(server,queue,voname='',algorithm='longest',debug=0) :

    if debug: print 'running for VO', voname

    if voname in ['',None] :
        print 'empty vo name found'
        est = 101
        return adjusted_ett(est)
    # from here on out we can be assured there is a VO specified
    # filters:

    jobsqueued_thisqueue_thisvo = {'queue': queue, 'group': voname,
                                   'state': 'queued'}
    jobsqueued_anyqueue_thisvo = {'group': voname, 'state': 'queued'}
    jobsrunning = {'state': 'running'}
    
    nq = server.nmatch(jobsqueued_thisqueue_thisvo)
    if debug: print 'found', nq, 'queued jobs in', queue, \
       'queue for VO', voname
    if nq > 0: # there are queued jobs for this VO
        est = _ALGS[algorithm](server,server.jobs_last_query())
    elif server.freeSlots > 0 :
        if debug: print 'found', server.freeSlots, 'free job slots'
        est = 0
    else :
        server.nmatch(jobsrunning) # get all running jobs
        est_mrr = ett_most_recent_run(server.now(),
                                      server.jobs_last_query(),
                                      debug)

        est = est_mrr
    return adjusted_ett(est)

def ett_most_recent_run(now,list,debug=0) :
    est = 999
    # find most recently run job
    youngest = list[0]
    for j in list:
        if j.get('start') > youngest.get('start'): youngest = j

    if debug : print now, youngest.get('start'), youngest
    timefilled = now - youngest.get('start')
    if debug : print 'most recent run', timefilled
    return timefilled
